// Copyright by Uri Siman, Delphi 2 main module, QR Excel (xls) export filter
unit ProXLS;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  QuickRep, ComCtrls;

const Siz = 50; //max count fields in row
      SizY = 40; //max (band) height of page in Pixels/ab1

type
   TOnPrintEvent = procedure(Sender : TObject; var Value : String) of object;
   TAfterPrintEvent = procedure(BandPrinted : Boolean) of object;
   TRepBeforePrintEvent = procedure(var ReportPrint : Boolean) of object;
   TRepAfterPrintEvent = procedure(Sender: TObject) of object;

SendPageXLSOn = Class
 OldCall : Array[0..Siz*5] of TOnPrintEvent;
 PrnBand : Array[0..Siz*5] of TAfterPrintEvent;
 end;

type
  TXLSParams = class (TPersistent)
   private
    FIgnorePageHeader : Boolean;
    FIgnorePageFooter : Boolean;
    FDefaultColWidth  : Word;
    FDefaultRowHeight : Word;
    FProtect          : Boolean;
    FFileName         : String;
    FFourDigitYear    : Boolean;
    FHeader           : String;
    FFooter           : String;
    FLeftMargin       : Byte;
    FRightMargin      : Byte;
    FTopMargin        : Byte;
    FBottomMargin     : Byte;
    FPrintHeaders     : Boolean;
    FPrintGridlines   : Boolean;
    FBackUp           : Boolean;
    FDefaultFont      : TFont;
   published
    property IgnorePageHeader : Boolean read FIgnorePageHeader write FIgnorePageHeader;
    property IgnorePageFooter : Boolean read FIgnorePageFooter write FIgnorePageFooter;
    property DefaultColWidth : Word read FDefaultColWidth write FDefaultColWidth;
    property DefaultRowHeight : Word read FDefaultRowHeight write FDefaultRowHeight;
    property Protect : Boolean read FProtect write FProtect;
    property FileName : String read FFileName write FFileName;
    property FourDigitYear : Boolean read FFourDigitYear write FFourDigitYear;
    property Header : String read FHeader write FHeader;
    property Footer : String read FFooter write FFooter;
    property LeftMargin : Byte read FLeftMargin write FLeftMargin;
    property RightMargin : Byte read FRightMargin write FRightMargin;
    property TopMargin : Byte read FTopMargin write FTopMargin;
    property BottomMargin : Byte read FBottomMargin write FBottomMargin;
    property PrintHeaders : Boolean read FPrintHeaders write FPrintHeaders;
    property PrintGridlines : Boolean read FPrintGridlines write FPrintGridlines;
    property BackUp : Boolean read FBackUp write FBackUp;
    property DefaultFont : TFont read FDefaultFont write FDefaultFont;
  end;

type
  TXLSWindow = class (TPersistent)
   private
    FLeft         : Word;
    FTop          : Word;
    FWidth        : Word;
    FHeight       : Word;
    FVisible      : Boolean;
    FFormulas     : Boolean;
    FGridLines    : Boolean;
    FHeaders      : Boolean;
    FFreeze       : Boolean;
    FShowZero     : Boolean;
    FTopRow       : Word;
    FLeftMost     : Word;
    FDefaultColor : Boolean;
    FColor        : LongInt;
   published
    property Left : Word read FLeft write FLeft;
    property Top : Word read FTop write FTop;
    property Width : Word read FWidth write FWidth;
    property Height : Word read FHeight write FHeight;
    property Visible : Boolean read FVisible write FVisible;
    property Formulas : Boolean read FFormulas write FFormulas;
    property GridLines : Boolean read FGridLines write FGridLines;
    property Headers : Boolean read FHeaders write FHeaders;
    property Freeze : Boolean read FFreeze write FFreeze;
    property ShowZero : Boolean read FShowZero write FShowZero;
    property TopRow : Word read FTopRow write FTopRow;
    property LeftMost : Word read FLeftMost write FLeftMost;
    property DefaultColor : Boolean read FDefaultColor write FDefaultColor;
    property Color : LongInt read FColor write FColor;
  end;

type
  TXLSPane = class (TPersistent)
   private
     FHorSplit : Word;
     FVerSplit : Word;
     FTopRow   : Word;
     FLeftmost : Word;
     FNumber   : Byte;
   published
    property HorSplit : Word read FHorSplit write FHorSplit default 0;
    property VerSplit : Word read FVerSplit write FVerSplit default 0;
    property TopRow : Word read FTopRow write FTopRow default 0;
    property Leftmost : Word read FLeftmost write FLeftmost default 0;
    property Number : Byte read FNumber write FNumber default 0;
  end;

type
  TStencil = class (TPersistent)
   private
    FHorizontalSensitiv : Word;
    FVerticalSensitiv : Word;
   published
    property VerticalSensitiv : Word read FVerticalSensitiv write FVerticalSensitiv default 14;
    property HorizontalSensitiv : Word read FHorizontalSensitiv write FHorizontalSensitiv default 3;
  end;

type
  TTabBorders = class (TPersistent)
   private
    FBorderInside : Boolean;
    FBorderOutside : Boolean;
    FDetailToTable : Boolean;
    FTitleToTable : Boolean;
    FPageHeaderToTable : Boolean;
    FPageFooterToTable : Boolean;
    FGroupHeaderToTable : Boolean;
    FGroupFooterToTable : Boolean;
    FSubDetailToTable : Boolean;
    FColumnHeaderToTable : Boolean;
    FOverlayToTable : Boolean;
    FSummaryToTable : Boolean;
   published
    property BorderInside : Boolean read FBorderInside write FBorderInside;
    property BorderOutside : Boolean read FBorderOutside write FBorderOutside;
    property DetailToTable : Boolean read FDetailToTable write FDetailToTable;
    property TitleToTable : Boolean read FTitleToTable write FTitleToTable;
    property PageHeaderToTable : Boolean read FPageHeaderToTable write FPageHeaderToTable;
    property PageFooterToTable : Boolean read FPageFooterToTable write FPageFooterToTable;
    property GroupHeaderToTable : Boolean read FGroupHeaderToTable write FGroupHeaderToTable;
    property GroupFooterToTable : Boolean read FGroupFooterToTable write FGroupFooterToTable;
    property SubDetailToTable : Boolean read FSubDetailToTable write FSubDetailToTable;
    property ColumnHeaderToTable : Boolean read FColumnHeaderToTable write FColumnHeaderToTable;
    property OverlayToTable : Boolean read FOverlayToTable write FOverlayToTable;
    property SummaryToTable : Boolean read FSummaryToTable write FSummaryToTable;
  end;


  TDRepXLS = class(TComponent)
  private
    { Private declarations }
    FExecute : Boolean;
    FXLSParams : TXLSParams;
    FXLSWindow : TXLSWindow;
    FXLSPane : TXLSPane;
    FStencil : TStencil;
    FTabBorders : TTabBorders;
    FShowProgress : Boolean;
  protected
    { Protected declarations }
    procedure SetExecute(Value : boolean);
  public
    { Public declarations }
    PageXLSOn : SendPageXLSOn;
    procedure QRLabel1Print(sender: TObject; var Value: string);
    procedure QRBand1AfterPrint(BandPrinted: Boolean);
    procedure APrint(sender: TObject);
    procedure BPrint(var PrintReport : Boolean);
    procedure Loaded; override;
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
    property Execute : Boolean read FExecute write SetExecute;
    property XLSParams : TXLSParams read FXLSParams write FXLSParams;
    property XLSWindow : TXLSWindow read FXLSWindow write FXLSWindow;
//    property XLSPane : TXLSPane read FXLSPane write FXLSPane;
    property Stencil : TStencil read FStencil write FStencil;
    property TabBorders : TTabBorders read FTabBorders write FTabBorders;
    property ShowProgress : Boolean read FShowProgress write FShowProgress;
  end;

var Bevent : Boolean;
       tqr : TQuickReport;
   fm, gfm : TForm;
       mpb : TProgressBar;
       Ab1 : Word; {14 : Vertical Sensitivity in Pixels}
       Ab2 : Word; {10 : Horizontal Sensitivity in Pixels}
   RowSent : Word;
    OldRef : Word;

procedure Register;

Function PagePack(Y, L : Integer) : Word;

implementation
uses ProSXLS;

Var
   BefPrn : TRepBeforePrintEvent;
   AftPrn : TRepAfterPrintEvent;

constructor TDRepXLS.Create(AOwner : TComponent);
begin
 inherited Create(AOwner);
 FStencil:=TStencil.Create;
 FXLSParams:=TXLSParams.Create;
 FXLSWindow:=TXLSWindow.Create;
 FXLSPane:=TXLSPane.Create;
 FTabBorders:=TTabBorders.Create;

 With FStencil Do
 Begin
  FVerticalSensitiv:= 14;
  FHorizontalSensitiv:= 10;
 end;

 With FXLSWindow Do
 Begin
  FLeft         := 0;
  FTop          := 0;
  FWidth        := 600;
  FHeight       := 400;
  FVisible      := True;
  FFormulas     := False;
  FGridLines    := True;
  FHeaders      := True;
  FFreeze       := False;
  FShowZero     := True;
  FTopRow       := 1;
  FLeftMost     := 1;
  FDefaultColor := True;
  FColor        := 0;
 end;

 With FXLSParams Do
 Begin
  FIgnorePageHeader:= True;
  FIgnorePageFooter:= True;
  FDefaultColWidth := 12;
  FDefaultRowHeight:= 13;
  FProtect:= False;
  FFileName:= 'Report.xls';
  FFourDigitYear:= True;
  FHeader:='';
  FFooter:='';
  FLeftMargin:=0;
  FRightMargin:=0;
  FTopMargin:=0;
  FBottomMargin:=0;
  FPrintHeaders:=False;
  FPrintGridlines:=False;
  FBackUp:=False;
  FDefaultFont:=TFont.Create;
  With FDefaultFont Do
  begin
   Name:='Arial';
   Height:=13;
   Size:= 10;
  end;
 end;

 With FXLSPane Do
 Begin
  FHorSplit :=0;
  FVerSplit :=0;
  FTopRow   :=1;
  FLeftmost :=1;
  FNumber   :=0;
 end;


 With FTabBorders Do
 Begin
  FBorderInside:= True;
  FBorderOutSide:= True;
  FDetailToTable:= True;
  FTitleToTable:= False;
  FPageHeaderToTable:= False;
  FPageFooterToTable:= False;
  FGroupHeaderToTable:= False;
  FGroupFooterToTable:= False;
  FSubDetailToTable:= False;
  FColumnHeaderToTable:= False;
  FOverlayToTable:= False;
  FSummaryToTable:= False;
 end;

 OldRef :=0; //nothing band before
 PrnPage :=0; //printed pages

end;

destructor TDRepXLS.Destroy;
begin
 QRPrinter.CleanUp;

 FStencil.Free;
 FXLSParams.Free;
 FTabBorders.Free;

 inherited Destroy;
end;

procedure TDRepXLS.SetExecute(Value : boolean);
var frm : TComponent;
      i : Integer;
begin
If Value<>FExecute then
begin
    FExecute:=Value;

If FExecute then
begin
 RowSent:=0;

 Ab1:=FStencil.FVerticalSensitiv;
 Ab2:=FStencil.FHorizontalSensitiv;

 EXBP:=False;
 EXAP:=False;
 BEvent:=TRUE;
 frm:=Owner;
 while (not (frm is TForm)) do frm:=frm.Owner;
 fm:=(frm as TForm);
 with fm do
 begin
  i:=0;
  while (i<ComponentCount) do
    if (Components[i] is TQuickReport) then
     begin
      tqr:=(Components[i] as TQuickReport);
      If Assigned((Components[i] as TQuickReport).BeforePrint) then
         EXBP:=True; {saving old call BeforePrint}
      If Assigned((Components[i] as TQuickReport).AfterPrint) then
         EXAP:=True; {saving old call AfterPrint}

      AftPrn:=tqr.AfterPrint;
      BefPrn:=tqr.BeforePrint;
      tqr.BeforePrint:=BPrint;
      tqr.AfterPrint:=APrint;

      SubFFileName := FXLSParams.FFileName;

      tqr.Prepare;
      FExecute:=False;

      Break;
     end
    else Inc(i);
 end;
end;
end;
end;

Function PagePack(Y, L : Integer) : Word;
Var I, J : Integer;
Begin
 For I:=0 to PageXLS.Empty[Y]-1 Do
 If PageXLS.Left[I, Y] >= L then
 Begin
   For J:=PageXLS.Empty[Y] Downto I+1 Do
   Begin
    PageXLS.WLine[J, Y]:= PageXLS.WLine[J-1, Y];
    PageXLS.Top[J, Y]:= PageXLS.Top[J-1, Y];
    PageXLS.Left[J, Y]:= PageXLS.Left[J-1, Y];
    PageXLS.Width[J, Y]:= PageXLS.Width[J-1, Y];
    PageXLS.Align[J, Y]:= PageXLS.Align[J-1, Y];
    PageXLS.FName[J, Y]:= PageXLS.FName[J-1, Y];
    PageXLS.FSize[J, Y]:= PageXLS.FSize[J-1, Y];
    PageXLS.FStyle[J, Y]:= PageXLS.FStyle[J-1, Y];
   end;

   If PageXLS.Empty[Y]<Siz then
      PageXLS.Empty[Y]:=PageXLS.Empty[Y]+1;
   Result:=I;
   Exit;
 end;

 If PageXLS.Empty[Y]<Siz then
    PageXLS.Empty[Y]:=PageXLS.Empty[Y]+1;

 Result:=PageXLS.Empty[Y] - 1;
end;

procedure TDRepXLS.QRLabel1Print(sender: TObject; var Value: string);
var str : string;
   X, Y : Integer;
    Pix : Integer;
     Tg : LongInt;
begin

If Length(Value)<=0 then
If ((TQRBand((Sender As Tcontrol).Parent).BandType=rbTitle)
  And (FTabBorders.FTitleToTable=True)) Or
   ((TQRBand((Sender As Tcontrol).Parent).BandType=rbDetail)
  And (FTabBorders.FDetailToTable=True)) Or
   ((TQRBand((Sender As Tcontrol).Parent).BandType=rbPageHeader)
  And (FTabBorders.FPageHeaderToTable=True)) Or
   ((TQRBand((Sender As Tcontrol).Parent).BandType=rbPageFooter)
  And (FTabBorders.FPageFooterToTable=True)) Or
   ((TQRBand((Sender As Tcontrol).Parent).BandType=rbGroupHeader)
  And (FTabBorders.FGroupHeaderToTable=True)) Or
   ((TQRBand((Sender As Tcontrol).Parent).BandType=rbGroupFooter)
  And (FTabBorders.FGroupFooterToTable=True)) Or
   ((TQRBand((Sender As Tcontrol).Parent).BandType=rbSubDetail)
  And (FTabBorders.FSubDetailToTable=True)) Or
   ((TQRBand((Sender As Tcontrol).Parent).BandType=rbColumnHeader)
  And (FTabBorders.FColumnHeaderToTable=True)) Or
   ((TQRBand((Sender As Tcontrol).Parent).BandType=rbOverlay)
  And (FTabBorders.FOverlayToTable=True)) Or
   ((TQRBand((Sender As Tcontrol).Parent).BandType=rbSummary)
  And (FTabBorders.FSummaryToTable=True)) then
  Value:=' ';

 Case PageXLS.Typ[(Sender As TComponent).Tag] of
 1: If Assigned(PageXLSOn.OldCall[(Sender As TQRLabel).Tag]) then
    Begin
     (Sender As TQRLabel).OnPrint:=
                 PageXLSOn.OldCall[(Sender As TQRLabel).Tag];
      Tg:=(Sender As TQRLabel).Tag;
     (Sender As TQRLabel).Tag:= PageXLS.OldTag[Tg];
     (Sender As TQRLabel).OnPrint(sender, Value);
     (Sender As TQRLabel).Tag:=Tg;
    end;
 2: If Assigned(PageXLSOn.OldCall[(Sender As TQRDBText).Tag]) then
    Begin
     (Sender As TQRDBText).OnPrint:=
                 PageXLSOn.OldCall[(Sender As TQRDBText).Tag];
      Tg:=(Sender As TQRDBText).Tag;
     (Sender As TQRDBText).Tag:= PageXLS.OldTag[Tg];
     (Sender As TQRDBText).OnPrint(sender, Value);
     (Sender As TQRDBText).Tag:=Tg;
    end;
 3: If Assigned(PageXLSOn.OldCall[(Sender As TQRDBCalc).Tag]) then
    Begin
     (Sender As TQRDBCalc).OnPrint:=
                 PageXLSOn.OldCall[(Sender As TQRDBCalc).Tag];
      Tg:=(Sender As TQRDBCalc).Tag;
     (Sender As TQRDBCalc).Tag:= PageXLS.OldTag[Tg];
     (Sender As TQRDBCalc).OnPrint(sender, Value);
     (Sender As TQRDBCalc).Tag:=Tg;
    end;
 4: If Assigned(PageXLSOn.OldCall[(Sender As TQRSysData).Tag]) then
    Begin
     (Sender As TQRSysData).OnPrint:=
                 PageXLSOn.OldCall[(Sender As TQRSysData).Tag];
      Tg:=(Sender As TQRSysData).Tag;
     (Sender As TQRSysData).Tag:= PageXLS.OldTag[Tg];
     (Sender As TQRSysData).OnPrint(sender, Value);
     (Sender As TQRSysData).Tag:=Tg;
    end;
  end;

  Ref:=PageXLS.Band[(Sender As TComponent).Tag];

  If (Sender Is TQRLabel) then
  Begin
   Y:= (Sender As TQRLabel).Top div Ab1;
   X:= (Sender As TQRLabel).Left div Ab2;
   PageXLS.Name[(Sender As TComponent).Tag]:=
                 (Sender As TQRLabel).Name;

   X:=PagePack(Y, (Sender As TQRLabel).Left);

   PageXLS.WLine[X, Y]:= Value;
   PageXLS.Top[X, Y]:= (Sender As TQRLabel).Top;
   PageXLS.Left[X, Y]:= (Sender As TQRLabel).Left;
   PageXLS.Width[X, Y]:= (Sender As TQRLabel).Width;
   PageXLS.Align[X, Y]:= (Sender As TQRLabel).Alignment;
   PageXLS.FName[X, Y]:= (Sender As TQRLabel).Font.Name;
   PageXLS.FSize[X, Y]:= (Sender As TQRLabel).Font.Size;
   PageXLS.FStyle[X, Y]:= (Sender As TQRLabel).Font.Style;
   end;

  If (Sender Is TQRDBText) then
  Begin
   {   }
   Y:= (Sender As TQRDBText).Top div Ab1;
   X:= (Sender As TQRDBText).Left div Ab2;
   PageXLS.Name[(Sender As TComponent).Tag]:=
                 (Sender As TQRDBText).Name;

   X:=PagePack(Y, (Sender As TQRDBText).Left);

   PageXLS.WLine[X, Y]:= Value;
   PageXLS.Top[X, Y]:= (Sender As TQRDBText).Top;
   PageXLS.Left[X, Y]:= (Sender As TQRDBText).Left;
   PageXLS.Width[X, Y]:= (Sender As TQRDBText).Width;
   PageXLS.Align[X, Y]:= (Sender As TQRDBText).Alignment;
   PageXLS.FName[X, Y]:= (Sender As TQRDBText).Font.Name;
   PageXLS.FSize[X, Y]:= (Sender As TQRDBText).Font.Size;
   PageXLS.FStyle[X, Y]:= (Sender As TQRDBText).Font.Style;
  end;

  If (Sender Is TQRDBCalc) then
  Begin
   Y:= (Sender As TQRDBCalc).Top div Ab1;
   X:= (Sender As TQRDBCalc).Left div Ab2;
   PageXLS.Name[(Sender As TComponent).Tag]:=
                 (Sender As TQRDBCalc).Name;

   X:=PagePack(Y, (Sender As TQRDBCalc).Left);

   PageXLS.WLine[X, Y]:= Value;
   PageXLS.Top[X, Y]:= (Sender As TQRDBCalc).Top;
   PageXLS.Left[X, Y]:= (Sender As TQRDBCalc).Left;
   PageXLS.Width[X, Y]:= (Sender As TQRDBCalc).Width;
   PageXLS.Align[X, Y]:= (Sender As TQRDBCalc).Alignment;
   PageXLS.FName[X, Y]:= (Sender As TQRDBCalc).Font.Name;
   PageXLS.FSize[X, Y]:= (Sender As TQRDBCalc).Font.Size;
   PageXLS.FStyle[X, Y]:= (Sender As TQRDBCalc).Font.Style;
   end;

  If (Sender Is TQRSysData) then
  Begin
   Y:= (Sender As TQRSysData).Top div Ab1;
   X:= (Sender As TQRSysData).Left div Ab2;
   PageXLS.Name[(Sender As TComponent).Tag]:=
                 (Sender As TQRSysData).Name;

   X:=PagePack(Y, (Sender As TQRSysData).Left);

   PageXLS.WLine[X, Y]:= Value;
   PageXLS.Top[X, Y]:= (Sender As TQRSysData).Top;
   PageXLS.Left[X, Y]:= (Sender As TQRSysData).Left;
   PageXLS.Width[X, Y]:= (Sender As TQRSysData).Width;
   PageXLS.Align[X, Y]:= (Sender As TQRSysData).Alignment;
   PageXLS.FName[X, Y]:= (Sender As TQRSysData).Font.Name;
   PageXLS.FSize[X, Y]:= (Sender As TQRSysData).Font.Size;
   PageXLS.FStyle[X, Y]:= (Sender As TQRSysData).Font.Style;
   end;
  try
   RefW:=TQRBand((Sender As Tcontrol).Parent).Height;
  except
   RefW:=50000; //band not exist for this control
  end;
end;

procedure TDRepXLS.QRBand1AfterPrint(BandPrinted: Boolean);
begin
   SubFHorizontalSensitiv:= FStencil.FHorizontalSensitiv;

   SubFTitleToTable:=FTabBorders.FTitleToTable;
   SubFDetailToTable:=FTabBorders.FDetailToTable;
   SubFPageHeaderToTable:=FTabBorders.FPageHeaderToTable;
   SubFPageFooterToTable:=FTabBorders.FPageFooterToTable;
   SubFGroupHeaderToTable:=FTabBorders.FGroupHeaderToTable;
   SubFGroupFooterToTable:=FTabBorders.FGroupFooterToTable;
   SubFSubDetailToTable:=FTabBorders.FSubDetailToTable;
   SubFColumnHeaderToTable:=FTabBorders.FColumnHeaderToTable;
   SubFOverlayToTable:=FTabBorders.FOverlayToTable;
   SubFSummaryToTable:=FTabBorders.FSummaryToTable;

   SubIgnorePageHeader:=FXLSParams.FIgnorePageHeader;
   SubIgnorePageFooter:=FXLSParams.FIgnorePageFooter;
   SubFRowHeight:=FXLSParams.FDefaultRowHeight * 20;
   SubFColWidth:=FXLSParams.FDefaultColWidth * 7;

   SubBorderInside:=FTabBorders.BorderInside;
   SubBorderOutside:=FTabBorders.BorderOutside;

   OnAftPrn; {calling transfer method}

   If (Ref=2) and FShowProgress then
   With Mpb Do
     If Position + Step <= Max then
     begin
        Position:=Position + Step;
        Show;
     end;

end;

procedure TDRepXLS.APrint(sender : TObject);
var   i : integer;
begin

 if FExecute=False then Exit;

 xlsFileClose;

 Gfm.Free;

 I:=0;
 with fm do
 while (i<ComponentCount) do
 if (Components[i] is TQuickReport) then
 begin {calling old afterprint}
    (Components[i] As TQuickReport).AfterPrint:=AftPrn;
    If EXAP then
       (Components[i] As TQuickReport).AfterPrint(sender);
    Break;
 end
 else Inc(i);

end;


procedure TDRepXLS.Loaded;
begin

 inherited Loaded;

end;

procedure TDRepXLS.BPrint(var PrintReport : Boolean);
Var I, J : Integer;
str,str1 : string;
begin

 if FExecute=False then Exit;

  I:=0;
  with fm do
  while (i<ComponentCount) do
  if (Components[i] is TQuickReport) then
  begin {  beforeprint}
    (Components[i] As TQuickReport).BeforePrint:=BefPrn;
    If EXBP then
       (Components[i] As TQuickReport).BeforePrint(PrintReport);

 Gfm:= TForm.Create(Application);
 If FShowProgress then
 begin
 With Gfm Do
 Begin
  Height:=50;
  Width:=300;
  BorderStyle:=bsSingle;
  Caption:='Progress...';
  Position:=poScreenCenter;
 end;

 Mpb:=TProgressBar.Create(nil);
 Mpb.Parent:=Gfm;
 With Mpb Do
 Begin
  Align:=alClient;
  Step:=1;
  Max:= (fm.Components[I] As TQuickReport).DataSource.DataSet.RecordCount;
  Position:=0;
  Visible:=True;
 end;
  Gfm.Show;
  Gfm.BringToFront;
  Gfm.SetFocus;
 end;

    Break;
  end
  else Inc(i);

  if Not BEvent then exit;
  if  FExecute then
  begin

   Ref:=0;
   L:=0;

  try
   xlsFileOpen;
  except
   ShowMessage('Sorry!.. Can`t create XLS file!!!');
   exit;
  end;

   If  FXLSParams.FProtect then xlsProtect(1)
                           else xlsProtect(0);
   xlsLang;

   xlsRefMode(1);

   xlsMillenium(FXLSParams.FFourDigitYear);

   xlsHeader(FXLSParams.FHeader);
   xlsFooter(FXLSParams.FFooter);

   xlsLeftMargin(FXLSParams.FLeftMargin / 35.0);
   xlsRightMargin(FXLSParams.FRightMargin / 35.0);
   xlsTopMargin(FXLSParams.FTopMargin / 35.0);
   xlsBottomMargin(FXLSParams.FBottomMargin / 35.0);

   xlsPrintGridlines(FXLSParams.FPrintGridlines);
   xlsPrintHeaders(FXLSParams.FPrintHeaders);

   xlsWindowPosition(FXLSWindow.FLeft * 20, FXLSWindow.FTop * 20,
                     FXLSWindow.FWidth * 20, FXLSWindow.FHeight * 20,
                     Not(FXLSWindow.FVisible));

   xlsWindowParams(FXLSWindow.FFormulas, FXLSWindow.FGridLines,
                   FXLSWindow.FHeaders, FXLSWindow.FFreeze,
                   FXLSWindow.FShowZero, FXLSWindow.FTopRow - 1,
                   FXLSWindow.FLeftMost - 1, FXLSWindow.FDefaultColor,
                   FXLSWindow.FColor);

   xlsBackUp(FXLSParams.FBackUp);

 {  xlsPanes(FXLSPane.FHorSplit * 20, FXLSPane.FVerSplit * 20,
            FXLSPane.FTopRow, FXLSPane.FLeftmost,
            FXLSPane.Number);
 }

   xlsCellFont(FXLSParams.FDefaultFont.Size * 20,
               FXLSParams.FDefaultFont.Style=[fsBold],
               FXLSParams.FDefaultFont.Style=[fsItalic],
               FXLSParams.FDefaultFont.Style=[fsUnderline],
               FXLSParams.FDefaultFont.Style=[fsStrikeOut],
               FXLSParams.FDefaultFont.Name);


  Gfm.Canvas.Font:= FXLSParams.FDefaultFont;


   xlsColWidth(1, 255, FXLSParams.FDefaultColWidth * 192);

//   xlsName('Bo-bo');

   PageXLS:=SendPageXLS.Create;
   PageXLSOn:=SendPageXLSOn.Create;

   J:=0;

   For I:=0 to fm.ComponentCount - 1 Do
   Begin
    If (fm.Components[I] Is TQRBand) then
    Begin
     PageXLSOn.PrnBand[J]:=(fm.Components[I] As TQRBand).AfterPrint;
     (fm.Components[I] As TQRBand).AfterPrint:=QRBand1AfterPrint;
    end;

    If (fm.Components[I] Is TQRLabel) then
    Begin
     PageXLS.OldTag[J]:=fm.Components[I].Tag;
     PageXLS.Typ[J]:=1;
     Case ((fm.Components[I] As TQRLabel).Parent As TQRBand).BandType of
      rbTitle        : PageXLS.Band[J]:=1;
      rbDetail       : PageXLS.Band[J]:=2;
      rbPageHeader   : PageXLS.Band[J]:=3;
      rbPageFooter   : PageXLS.Band[J]:=4;
      rbGroupHeader  : PageXLS.Band[J]:=5;
      rbGroupFooter  : PageXLS.Band[J]:=6;
      rbSubDetail    : PageXLS.Band[J]:=7;
      rbColumnHeader : PageXLS.Band[J]:=8;
      rbOverlay      : PageXLS.Band[J]:=9;
      rbSummary      : PageXLS.Band[J]:=10;
     else
      PageXLS.Band[J]:=10;
     end;
     fm.Components[I].Tag:=J; {Labels}
     PageXLSOn.OldCall[J]:=TQRLabel(fm.Components[I]).OnPrint;
     TQRLabel(fm.Components[I]).OnPrint:=QRLabel1Print;
     Inc(J);
    end;
    If (fm.Components[I] Is TQRDBText) then
    Begin
     PageXLS.OldTag[J]:=fm.Components[I].Tag;
     PageXLS.Typ[J]:=2;
     Case ((fm.Components[I] As TQRDBText).Parent As TQRBand).BandType of
      rbTitle        : PageXLS.Band[J]:=1;
      rbDetail       : PageXLS.Band[J]:=2;
      rbPageHeader   : PageXLS.Band[J]:=3;
      rbPageFooter   : PageXLS.Band[J]:=4;
      rbGroupHeader  : PageXLS.Band[J]:=5;
      rbGroupFooter  : PageXLS.Band[J]:=6;
      rbSubDetail    : PageXLS.Band[J]:=7;
      rbColumnHeader : PageXLS.Band[J]:=8;
      rbOverlay      : PageXLS.Band[J]:=9;
      rbSummary      : PageXLS.Band[J]:=10;
     else
      PageXLS.Band[J]:=10;
     end;
     fm.Components[I].Tag:=J; {DBTexts}
     PageXLSOn.OldCall[J]:=TQRDBText(fm.Components[I]).OnPrint;
     TQRDBText(fm.Components[I]).OnPrint:=QRLabel1Print;
     Inc(J);
    end;
    If (fm.Components[I] Is TQRDBCalc) then
    Begin
     PageXLS.OldTag[J]:=fm.Components[I].Tag;
     PageXLS.Typ[J]:=3;
     Case ((fm.Components[I] As TQRDBCalc).Parent As TQRBand).BandType of
      rbTitle        : PageXLS.Band[J]:=1;
      rbDetail       : PageXLS.Band[J]:=2;
      rbPageHeader   : PageXLS.Band[J]:=3;
      rbPageFooter   : PageXLS.Band[J]:=4;
      rbGroupHeader  : PageXLS.Band[J]:=5;
      rbGroupFooter  : PageXLS.Band[J]:=6;
      rbSubDetail    : PageXLS.Band[J]:=7;
      rbColumnHeader : PageXLS.Band[J]:=8;
      rbOverlay      : PageXLS.Band[J]:=9;
      rbSummary      : PageXLS.Band[J]:=10;
     else
      PageXLS.Band[J]:=10;
     end;
     fm.Components[I].Tag:=J; {DBCalc}
     PageXLSOn.OldCall[J]:=TQRDBCalc(fm.Components[I]).OnPrint;
     TQRDBCalc(fm.Components[I]).OnPrint:=QRLabel1Print;
     Inc(J);
    end;
    If (fm.Components[I] Is TQRSysData) then
    Begin
     PageXLS.OldTag[J]:=fm.Components[I].Tag;
     PageXLS.Typ[J]:=4;
     Case ((fm.Components[I] As TQRSysData).Parent As TQRBand).BandType of
      rbTitle        : PageXLS.Band[J]:=1;
      rbDetail       : PageXLS.Band[J]:=2;
      rbPageHeader   : PageXLS.Band[J]:=3;
      rbPageFooter   : PageXLS.Band[J]:=4;
      rbGroupHeader  : PageXLS.Band[J]:=5;
      rbGroupFooter  : PageXLS.Band[J]:=6;
      rbSubDetail    : PageXLS.Band[J]:=7;
      rbColumnHeader : PageXLS.Band[J]:=8;
      rbOverlay      : PageXLS.Band[J]:=9;
      rbSummary      : PageXLS.Band[J]:=10;
     else
      PageXLS.Band[J]:=10;
     end;
     fm.Components[I].Tag:=J; {SysData}
     PageXLSOn.OldCall[J]:=TQRSysData(fm.Components[I]).OnPrint;
     TQRSysData(fm.Components[I]).OnPrint:=QRLabel1Print;
     Inc(J);
    end;
   end;
  end;
 BEvent:=FALSE;
end;

procedure Register;
begin
  RegisterComponents('QReport', [TDRepXLS]);
end;

end.
